from . import WeapiCryptoRequest,LoginRequiredException,GetCurrentSession

@WeapiCryptoRequest
def GetPlaylistInfo(playlist_id,offset=0,total=True,limit=1000):
    '''Fetches a playlist's content'''
    return '/weapi/v6/playlist/detail',{"id":str(playlist_id),"offset":str(offset),"total":str(total).lower(),"limit":str(limit)}

@WeapiCryptoRequest
def GetUserPlaylists(user_id=0, offset=0, limit=1001):
    '''
        Fetches a user's playlists.No VIP Level needed.

            user_id :   The ID of the user.Set 0 for yourself (if logged in)
            offset  :   sets where the comment begins
            limit   :   sets how many of them can be sent
    '''
    if user_id == 0 and GetCurrentSession().login_info['success']:
        user_id = GetCurrentSession().login_info['content']['account']['id']
    elif user_id == 0:
        raise LoginRequiredException('A user ID or your login info is needed')
    return '/weapi/user/playlist',{'offset': str(offset), 'limit': str(limit),'uid': str(user_id)}


'''
# Netease Cloud Music Core

Core class for handling NCM API Calls
'''

from pyncm.ncm.crypto import Crypto
from typing import Type
import requests,logging,time

class Session(requests.Session):
    '''
        # Session module
        Stores config and requests. Important things are at `apis`,take a look there
    '''
    HOST = "https://music.163.com"

    def __init__(self,*a,**k):
        # Setting up our request session
        super().__init__(*a,**k)
        self.headers = {
            "Content-Type": "application/x-www-form-urlencoded",
            "User-Agent": "mos9527 Him Self/v15",
            "Referer":Session.HOST
        }
        # Setting up default values
        self.login_info = {'success': False,'tick': time.time(), 'content': None}
        self.csrf_token = ''
        self.verify = False
        
        
class SessionManager():
    # How to abuse mutable types (((gone wild)))
    def __init__(self) -> None:
        self.session = Session()
    def get(self):
        return self.session
    def set(self,session):
        self.session = session

logger = logging.getLogger('ncm')
sessionManager    = SessionManager()
GetCurrentSession = sessionManager.get
SetCurrentSession = sessionManager.set
from .apis import *



from . import WeapiCryptoRequest
from . import logger
import json
@WeapiCryptoRequest
def GetTrackDetail(song_ids):
    '''
        Fetches a song's 'detailed' infomation

            song_ids        :        ID of songs (e.g. [1234,5678] or 1234)
    '''
    ids = song_ids if isinstance(song_ids,list) else [song_ids]
    return '/weapi/v3/song/detail',{"c":json.dumps([{'id': str(id)} for id in ids])}

@WeapiCryptoRequest
def GetTrackAudio(song_ids, quality='lossless',encodeType='aac'):
    '''
        Fetches a song's info.By default,it only returns the url and non-meta info.
        
        `quality` can be : standard,higher,lossless
        `encodeType` is yet unknown,though `aac` is valiad
    '''
    if not quality in ['standard', 'higher', 'lossless']:
        logger.warn('Invalid quality config `%s`,falling back to `standard`')
        quality = 'standard'
    # Quality check
    ids = song_ids if isinstance(song_ids,list) else [song_ids]
    return '/weapi/song/enhance/player/url/v1',{"ids":ids,"level":quality,"encodeType":encodeType}


@WeapiCryptoRequest
def GetTrackLyrics(song_id):
    '''Fetches a song's lyrics'''
    return '/weapi/song/lyric',{"id": str(song_id), "lv": -1, "tv": -1}


@WeapiCryptoRequest
def GetTrackComments(song_id, offset=0, limit=20):
    '''
        Fetches a song's comments

            offset  :   sets where the comment begins
            limit   :   sets how many of them can be sent
    '''
    return '/weapi/v1/resource/comments/R_SO_4_%s' % song_id,{"rid":str(song_id),"offset":str(offset),"total":"true","limit":str(limit)}


from . import WeapiCryptoRequest
from . import logger

@WeapiCryptoRequest
def GetAlbumInfo(album_id):
    '''Fetches an album's info'''
    return '/weapi/v1/album/%s' % album_id,{}

@WeapiCryptoRequest
def GetAlbumComments(album_id, offset=0, limit=20):
    '''Fetches a album's comments'''
    return '/weapi/v1/resource/comments/R_AL_3_%s' % album_id,{"rid":"R_AL_3_%s" % album_id,"offset":str(offset),"total":"true","limit":str(limit)}



from .. import GetCurrentSession
from .. import logger
from ..crypto import Crypto
import json

class LoginRequiredException(Exception):pass
class LoginFailedException(Exception):pass
# region Base models export
def BaseCryptoRequest(crypto):
    '''Base cryptographic request maker'''
    def preWrapper(func):
        def wrapper(*a,**k):
            url,plain = func(*a,**k)
            if isinstance(plain,dict):
                plain = json.dumps({**plain,'csrf_token':GetCurrentSession().csrf_token})
            if not isinstance(plain,str):plain = str(plain)
            # done converting,let's go
            payload = crypto(url,plain)
            req = GetCurrentSession().post(
                GetCurrentSession().HOST + url,
                params={'csrf_token':GetCurrentSession().csrf_token},
                data={
                    **payload
                }
            )
            rsp = req.text
            try:
                return json.loads(rsp)
            except json.JSONDecodeError:
                return rsp
        return wrapper
    return preWrapper

def WeapiCryptoRequest(func):
    return BaseCryptoRequest(lambda url,plain:Crypto.WeapiCrypto(plain))(func)

def LapiCryptoRequest(func):
    return BaseCryptoRequest(lambda url,plain:Crypto.LinuxCrypto(plain))(func)

def EapiCryptoRequest(func):
    return BaseCryptoRequest(lambda url,plain:Crypto.EapiCrypto(url,plain))(func)

# endregion

import os
__all__ = [i[:-3] for i in os.listdir(os.path.dirname(__file__)) if i[-2:] == 'py' and i != '__init__.py']
from . import * # import all local modules


from . import WeapiCryptoRequest

@WeapiCryptoRequest
def GetMVInfo(mv_id, res=1080):
    '''
        Fetches MV Info

            id      :   The ID of the MV
            res     :   MV Resolution
    '''
    return '/weapi/song/enhance/play/mv/url',{"id":str(mv_id),"r":str(res)}


@WeapiCryptoRequest
def GetMVComments(mv_id, offset=0, limit=20,total=True):
    '''
        Fetches a album's comments.No VIP Level needed.

            offset  :   sets where the comment begins
            limit   :   sets how many of them can be sent
    '''            
    return '/weapi/v1/resource/comments/R_MV_5_%s' % mv_id,{"rid":"R_MV_5_%s" % mv_id,"offset":str(offset),"total":str(total).lower(),"limit":str(limit)}


from . import WeapiCryptoRequest,GetCurrentSession,logger,Crypto,LoginFailedException
import time
def CellphoneLogin(phone='', password='',remeberLogin=True) -> dict:
    '''
        If given both phone number and password,updates them,and updates the cookies and the CSRF token

        Otherwise,if phone number and password are set,updates the cookies and the CSRF token

        Returns None if the response is invalid or the username & password combo is invalid

        Returns login info with structure tick/success/content that suggests the time/result/content of the attempt

        The value will be also set into the global variable to serve other functions
    '''
    sess = GetCurrentSession()
    if (phone and password):
        sess.phone = phone
        sess.password = password
        CellphoneLogin()
    else:
        if (sess.phone and sess.password):
            md5_password = Crypto.HashDigest(sess.password)
            r = WeapiCryptoRequest(lambda:('/weapi/login/cellphone',{"phone":str(sess.phone),"password":str(md5_password),"rememberLogin":str(remeberLogin).lower(),"checkToken":""}))()
            sess.login_info = {'tick': time.time(), 'content':r}
    
            if not sess.login_info['content']['code'] == 200:
                sess.login_info['success'] = False
                raise LoginFailedException(sess.login_info['content'])
            # HTTP Response code will always be 200.The REAL response code lies in the JSON.
            # 200 means login successful
            # 415 means the IP was reqeusting too frequently.This can be solved with a delayed request
            # 502 means the combo provided is wrong
            sess.login_info['success'] = True
            cookie = sess.cookies.get_dict()
            sess.csrf_token = cookie['__csrf']
            logger.debug('Updated login info for user %s' % sess.login_info['content']['profile']['nickname'])
            return sess.login_info
        else:
            raise LoginFailedException('Not enough login info provided')